@@ -11,12 +11,11 @@ import onepasswordconnectsdk
 from onepasswordconnectsdk.models import Item, ItemVault
 from onepasswordconnectsdk.models.constants import CONNECT_HOST_ENV_VARIABLE
 
-
 ENV_SERVICE_ACCOUNT_JWT_VARIABLE = "OP_CONNECT_TOKEN"
+UUIDLength = 26
 
 
 class Client:
-
     PRIMITIVE_TYPES = (float, bool, bytes, six.text_type) + six.integer_types
     NATIVE_TYPES_MAPPING = {
         "int": int,
@@ -44,10 +43,9 @@ class Client:
 
     def build_headers(self):
         """Builds the headers needed to make a request to the server
-        Parameters:
 
         Returns:
-        dict: The 1Password Connect API request headers
+            dict: The 1Password Connect API request headers
         """
 
         headers = {}
@@ -55,14 +53,91 @@ class Client:
         headers["Content-Type"] = "application/json"
         return headers
 
-    def get_item(self, item_id: str, vault_id: str):
-        """Get a specific item by uuid
-        Parameters:
-        item_id (str): The id of the item to be fetched
-        vault_id (str): The id of the vault in which to get the item from
+    def get_file(self, file_id: str, item_id: str, vault_id: str):
+        url = f"/v1/vaults/{vault_id}/items/{item_id}/files/{file_id}"
+        response = self.build_request("GET", url)
+        try:
+            response.raise_for_status()
+        except HTTPError:
+            raise FailedToRetrieveItemException(
+                f"Unable to retrieve item. Received {response.status_code}\
+                                     for {url} with message: {response.json().get('message')}"
+            )
+        return self.deserialize(response.content, "File")
+
+    def get_files(self, item_id: str, vault_id: str):
+        url = f"/v1/vaults/{vault_id}/items/{item_id}/files"
+
+        response = self.build_request("GET", url)
+        try:
+            response.raise_for_status()
+        except HTTPError:
+            raise FailedToRetrieveItemException(
+                f"Unable to retrieve item. Received {response.status_code}\
+                             for {url} with message: {response.json().get('message')}"
+            )
+        return self.deserialize(response.content, "list[File]")
+
+    def get_file_content(self, file_id: str, item_id: str, vault_id: str):
+        url = f"/v1/vaults/{vault_id}/items/{item_id}/files/{file_id}/content"
+
+        response = self.build_request("GET", url)
+        try:
+            response.raise_for_status()
+        except HTTPError:
+            raise FailedToRetrieveItemException(
+                f"Unable to retrieve items. Received {response.status_code} \
+                     for {url} with message: {response.json().get('message')}"
+            )
+        return response.content
+
+    def download_file(self, file_id: str, item_id: str, vault_id: str, path: str):
+        file_object = self.get_file(file_id, item_id, vault_id)
+        filename = file_object.name
+        content = self.get_file_content(file_id, item_id, vault_id)
+        global_path = os.path.join(path, filename)
+
+        file = open(global_path, "wb")
+        file.write(content)
+        file.close()
+
+    def get_item(self, item: str, vault: str):
+        """Get a specific item
+
+        Args:
+            item (str): the id or title of the item to be fetched
+            vault (str): the id or name of the vault in which to get the item from
+
+        Raises:
+            FailedToRetrieveItemException: Thrown when a HTTP error is returned
+            from the 1Password Connect API
 
         Returns:
-        Item object: The found item
+            Item object: The found item
+        """
+
+        vault_id = vault
+        if not self._is_valid_UUID(vault):
+            vault_id = self.get_vault_by_title(vault).id
+
+        if self._is_valid_UUID(item):
+            return self.get_item_by_id(item, vault_id)
+        else:
+            return self.get_item_by_title(item, vault_id)
+
+    def get_item_by_id(self, item_id: str, vault_id: str):
+        """Get a specific item by uuid
+       
+        Args:
+            item_id (str): The id of the item to be fetched
+            vault_id (str): The id of the vault in which to get the item from
+
+        Raises:
+            FailedToRetrieveItemException: Thrown when a HTTP error is returned
+            from the 1Password Connect API
+
+        Returns:
+            Item object: The found item
         """
         url = f"/v1/vaults/{vault_id}/items/{item_id}"
 
@@ -78,12 +153,17 @@ class Client:
 
     def get_item_by_title(self, title: str, vault_id: str):
         """Get a specific item by title
-        Parameters:
-        title (str): The title of the item to be fetched
-        vault_id (str): The id of the vault in which to get the item from
+        
+        Args:
+            title (str): The title of the item to be fetched
+            vault_id (str): The id of the vault in which to get the item from
+
+        Raises:
+            FailedToRetrieveItemException: Thrown when a HTTP error is returned
+            from the 1Password Connect API
 
         Returns:
-        Item object: A summary of the found item
+            Item object: The found item
         """
         filter_query = f'title eq "{title}"'
         url = f"/v1/vaults/{vault_id}/items?filter={filter_query}"
@@ -103,7 +183,8 @@ class Client:
                     title {title}"
             )
 
-        return self.deserialize(response.content, "list[SummaryItem]")[0]
+        item_summary = self.deserialize(response.content, "list[SummaryItem]")[0]
+        return self.get_item_by_id(item_summary.id, vault_id)
 
     def get_items(self, vault_id: str):
         """Returns a list of item summaries for the specified vault
@@ -235,6 +316,39 @@ class Client:
 
         return self.deserialize(response.content, "Vault")
 
+    def get_vault_by_title(self, name: str):
+        """Returns the vault with the given name
+        
+        Args:
+            name (str): The name of the vault in which to fetch
+        
+        Raises:
+            FailedToRetrieveVaultException: Thrown when a HTTP error is
+            returned from the 1Password Connect API
+
+        Returns:
+            Vault: The specified vault
+        """
+        filter_query = f'name eq "{name}"'
+        url = f"/v1/vaults?filter={filter_query}"
+
+        response = self.build_request("GET", url)
+        try:
+            response.raise_for_status()
+        except HTTPError:
+            raise FailedToRetrieveVaultException(
+                f"Unable to retrieve vaults. Received {response.status_code} \
+                     for {url} with message {response.json().get('message')}"
+            )
+
+        if len(response.json()) != 1:
+            raise FailedToRetrieveItemException(
+                f"Found {len(response.json())} vaults with \
+                    name {name}"
+            )
+
+        return self.deserialize(response.content, "list[Vault]")[0]
+
     def get_vaults(self):
         """Returns all vaults for service account set in client
 
@@ -442,8 +556,8 @@ class Client:
         """
         has_discriminator = False
         if (
-            hasattr(klass, "get_real_child_model")
-            and klass.discriminator_value_class_map
+                hasattr(klass, "get_real_child_model")
+                and klass.discriminator_value_class_map
         ):
             has_discriminator = True
 
@@ -452,9 +566,9 @@ class Client:
 
         kwargs = {}
         if (
-            data is not None
-            and klass.openapi_types is not None
-            and isinstance(data, (list, dict))
+                data is not None
+                and klass.openapi_types is not None
+                and isinstance(data, (list, dict))
         ):
             for attr, attr_type in six.iteritems(klass.openapi_types):
                 if klass.attribute_map[attr] in data:
@@ -469,6 +583,15 @@ class Client:
                 instance = self.__deserialize(data, klass_name)
         return instance
 
+    def _is_valid_UUID(self, uuid):
+        if len(uuid) is not UUIDLength:
+            return False
+        for c in uuid:
+            valid = (c >= 'a' and c <= 'z') or (c >= '0' and c <= '9')
+            if valid is False:
+                return False
+        return True
+
 
 def new_client(url: str, token: str):
     """Builds a new client for interacting with 1Password Connect
@@ -518,9 +641,11 @@ class OnePasswordConnectSDKError(RuntimeError):
 class EnvironmentTokenNotSetException(OnePasswordConnectSDKError, TypeError):
     pass
 
+
 class EnvironmentHostNotSetException(OnePasswordConnectSDKError, TypeError):
     pass
 
+
 class FailedToRetrieveItemException(OnePasswordConnectSDKError):
     pass
 
